package cn.bunny.security.filter;

import cn.bunny.common.result.Result;
import cn.bunny.common.utils.ResponseUtil;
import cn.bunny.enums.ResultCodeEnum;
import com.alibaba.fastjson2.JSON;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class TokenAuthenticationFilter extends OncePerRequestFilter {
    private final RedisTemplate<String, Object> redisTemplate;

    public TokenAuthenticationFilter(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws ServletException, IOException {
        // 如果是登录接口，直接放行
        if ("/admin/system/index/login".equals(request.getRequestURI())) {
            chain.doFilter(request, response);
            return;
        }

        UsernamePasswordAuthenticationToken authentication = getAuthentication(request);

        if (authentication != null) {
            SecurityContextHolder.getContext().setAuthentication(authentication);
            chain.doFilter(request, response);
        } else {
            ResponseUtil.out(response, Result.error(ResultCodeEnum.LOGIN_MOBLE_ERROR));
        }
    }

    private UsernamePasswordAuthenticationToken getAuthentication(HttpServletRequest request) {
        // 请求头是否有Token
        String token = request.getHeader("token");
        if (!StringUtils.hasText(token)) {
            Object authObject = redisTemplate.opsForValue().get(token);
            String authString = JSON.toJSONString(authObject);
            if (StringUtils.hasText(authString)) {
                List<Map> maplist = JSON.parseArray(authString, Map.class);
                System.out.println(maplist);
                List<SimpleGrantedAuthority> authList = new ArrayList<>();
                for (Map map : maplist) {
                    String authority = (String) map.get("roleList");
                    authList.add(new SimpleGrantedAuthority(authority));
                }
                return new UsernamePasswordAuthenticationToken(token, null, authList);
            } else {
                return new UsernamePasswordAuthenticationToken(token, null, new ArrayList<>());
            }
        }
        return null;
    }
}
